{!! auto center close button when width changes}

unit Adrbku1;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Menus,
  Adrbku2, Adrbku3,
  OvcBase, OvcData, OvcVlb, StdCtrls, Buttons;

type
  TfrmAddressBook = class(TForm)
    mnuMain: TMainMenu;
    miFile: TMenuItem;
    miNew: TMenuItem;
    miOpen: TMenuItem;
    miSave: TMenuItem;
    miSaveAs: TMenuItem;
    miExit: TMenuItem;
    miEdit: TMenuItem;
    miInsert: TMenuItem;
    miModify: TMenuItem;
    miDelete: TMenuItem;
    miHelp: TMenuItem;
    miAbout: TMenuItem;
    VLB: TOvcVirtualListbox;
    btnClose: TBitBtn;
    dlgOpen: TOpenDialog;
    dlgSaveAs: TSaveDialog;
    OvcController1: TOvcController;
    procedure miNewClick(Sender: TObject);
    procedure miOpenClick(Sender: TObject);
    procedure miSaveClick(Sender: TObject);
    procedure miSaveAsClick(Sender: TObject);
    procedure miExitClick(Sender: TObject);
    procedure miInsertClick(Sender: TObject);
    procedure miModifyClick(Sender: TObject);
    procedure miDeleteClick(Sender: TObject);
    procedure miAboutClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure VLBGetItem(Sender: TObject; Index: Longint;
      var ItemString: OpenString);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private declarations }

    procedure ClearData;
    procedure LoadData;
    procedure SaveData;
    procedure SaveDataAs;

  public
    { Public declarations }
    FileName : string;
    Modified : Boolean;
    Data : TList;
  end;

var
  frmAddressBook: TfrmAddressBook;

implementation

{$R *.DFM}

procedure TfrmAddressBook.ClearData;
var
  I : Integer;
begin
  for I := 0 to Data.Count-1 do
    FreeMem(Data[I], SizeOf(TAddressRec));
end;

procedure TfrmAddressBook.LoadData;
begin
{!!}
  Modified := False;
end;

procedure TfrmAddressBook.SaveData;
begin
{!!}
  Modified := False;
end;

procedure TfrmAddressBook.SaveDataAs;
begin
  if dlgSaveAs.Execute then begin
    FileName := dlgSaveAs.FileName;
    SaveData;
  end;
end;

procedure TfrmAddressBook.miNewClick(Sender: TObject);
begin
  FileName := '';
  ClearData;
  VLB.Repaint;
end;

procedure TfrmAddressBook.miOpenClick(Sender: TObject);
var
  Res : Integer;
begin
  Res := idNo;
  if Modified then
    Res := MessageDlg('File has been modified. Save current file?', mtWarning, mbYesNoCancel, 0);
  if Res = idYes then begin
  end else if Res = idNo then begin
    if dlgOpen.Execute then begin
      FileName := dlgOpen.FileName;
      ClearData;
      LoadData;
      VLB.Repaint;
    end;
  end;
end;

procedure TfrmAddressBook.miSaveClick(Sender: TObject);
begin
  if FileName = '' then
    SaveDataAs
  else
    SaveData;
end;

procedure TfrmAddressBook.miSaveAsClick(Sender: TObject);
begin
  SaveDataAs;
end;

procedure TfrmAddressBook.miExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmAddressBook.miInsertClick(Sender: TObject);
var
  A : PAddressRec;
begin
  GetMem(A, SizeOf(TAddressRec));
  frmEdit.ClearFields;
  OvcController1.MarkAsUninitialized(True);
  frmEdit.ShowModal;
  if frmEdit.ModalResult = idOK then begin
    frmEdit.GetFields(A);
    Data.Add(A);
    VLB.NumItems := Data.Count;

    if (VLB.ItemIndex < 0) and (Data.Count > 0) then
      VLB.ItemIndex := 0;

    VLB.Repaint;
    Modified := True;
  end else
    FreeMem(A, SizeOf(TAddressRec));
end;

procedure TfrmAddressBook.miModifyClick(Sender: TObject);
var
  L : LongInt;
  A : PAddressRec;
begin
  L := VLB.ItemIndex;
  if (L < 0) or (VLB.NumItems = 0) then Exit;
  A := Data[L];
  frmEdit.SetFields(A);
  OvcController1.MarkAsUninitialized(False);
  frmEdit.ShowModal;
  if frmEdit.ModalResult = idOK then begin
    frmEdit.GetFields(A);
    VLB.Repaint;
    Modified := True;
  end;
end;

procedure TfrmAddressBook.miDeleteClick(Sender: TObject);
var
  L : LongInt;
  A : PAddressRec;
begin
  L := VLB.ItemIndex;
  if (L < 0) or (VLB.NumItems = 0) then Exit;
  A := Data[L];
  if MessageDlg(
     Format('Delete record for %s, %s?', [A^.Last, A^.First]),
     mtConfirmation, [mbYes, mbNo], 0) = idYes then begin
    FreeMem(A, SizeOf(TAddressRec));
    Data.Delete(L);
    VLB.NumItems := Data.Count;
    VLB.Repaint;
    Modified := True;
  end;
end;

procedure TfrmAddressBook.miAboutClick(Sender: TObject);
begin
  frmAboutBox := TfrmAboutBox.Create(Self);
  try
    frmAboutBox.ShowModal;
  finally
    frmAboutBox.Free;
  end;
end;

procedure TfrmAddressBook.FormCreate(Sender: TObject);
begin
  Data := TList.Create;
  VLB.NumItems := 0;
end;

procedure TfrmAddressBook.FormDestroy(Sender: TObject);
begin
  ClearData;
  Data.Free;
end;

procedure TfrmAddressBook.VLBGetItem(Sender: TObject; Index: Longint;
  var ItemString: OpenString);
var
  A : PAddressRec;
begin
  if (Index >= 0) and (Index < Data.Count) then begin
    A := Data[Index];
    if A^.Company > '' then
      ItemString := Format('%s, %s (%s)', [A^.Last, A^.First, A^.Company])
    else
      ItemString := Format('%s, %s', [A^.Last, A^.First])
  end;
end;

procedure TfrmAddressBook.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if Modified then begin
    {data has been modified and ask to save}
    {if don't want to save the set modified to false}
    {otherwise, let save routine set modified}
{!!}
  end;
{!!  CanClose := Modified;}
end;

end.
